# © MNELAB developers
#
# License: BSD (3-clause)

from mnelab.dialogs.annotations import AnnotationsDialog
from mnelab.dialogs.append import AppendDialog
from mnelab.dialogs.calc import CalcDialog
from mnelab.dialogs.channel_properties import ChannelPropertiesDialog
from mnelab.dialogs.crop import CropDialog
from mnelab.dialogs.drop_bad_epochs import DropBadEpochsDialog
from mnelab.dialogs.epoch import EpochDialog
from mnelab.dialogs.erds import ERDSDialog, ERDSTopomapsDialog
from mnelab.dialogs.error_message import ErrorMessageBox
from mnelab.dialogs.events import EventsDialog
from mnelab.dialogs.filter import FilterDialog
from mnelab.dialogs.find_events import FindEventsDialog
from mnelab.dialogs.history import HistoryDialog
from mnelab.dialogs.interpolate_bads import InterpolateBadsDialog
from mnelab.dialogs.mat import MatDialog
from mnelab.dialogs.montage import MontageDialog
from mnelab.dialogs.npy import NpyDialog
from mnelab.dialogs.pick_channels import PickChannelsDialog
from mnelab.dialogs.plot_evoked import (
    PlotEvokedComparisonDialog,
    PlotEvokedDialog,
    PlotEvokedTopomaps,
)
from mnelab.dialogs.reference import ReferenceDialog
from mnelab.dialogs.rename_channels import RenameChannelsDialog
from mnelab.dialogs.run_ica import RunICADialog
from mnelab.dialogs.xdf_chunks import XDFChunksDialog
from mnelab.dialogs.xdf_metadata import XDFMetadataDialog
from mnelab.dialogs.xdf_streams import XDFStreamsDialog



import mne
import numpy as np
from mne.io import BaseRaw


class RawNPY(BaseRaw):
    """Raw data from .npy file."""

    def __init__(self, fname, fs, transpose=False):
        """Read raw data from .npy file.

        Parameters
        ----------
        fname : str
            File name to load.
        fs : float
            Sampling frequency (in Hz).
        transpose : bool
            Whether to transpose the data; set to `True` if the original shape is *not*
            (channels, samples).
        """
        data = np.load(fname)
        if transpose:
            data = data.T
        if data.ndim != 2:
            raise ValueError(f"Array must have two dimensions (got {data.ndim}).")
        info = mne.create_info(data.shape[0], fs)
        super().__init__(preload=data, info=info, filenames=[fname])


def read_raw_npy(fname, fs, transpose=False, *args, **kwargs):
    """Read raw data from .npy file.

    Parameters
    ----------
    fname : str
        File name to load.
    fs : float
        Sampling frequency (in Hz).
    transpose : bool
        Whether to transpose the data, the data should be of shape (channels, samples).

    Returns
    -------
    RawNPY
        The raw data.
    """
    return RawNPY(fname, fs, transpose)


def parse_npy(fname):
    """Return shape of array contained in .npy file.

    Parameters
    ----------
    fname : str
        File name to load.

    Returns
    -------
    shape : tuple[int, int]
        The shape of the array.
    """
    with open(fname, "rb") as f:
        np.lib.format.read_magic(f)
        shape, _, _ = np.lib.format.read_array_header_1_0(f)
    return shape



import pytest

from mnelab.io.readers import split_name_ext, supported


@pytest.mark.parametrize("ext", supported.keys())
def test_split_name_ext(ext):
    fname = f"test{ext}"
    assert split_name_ext(fname) == ("test", ext)


def test_split_name_ext_unsupported():
    assert split_name_ext("test.xxx") == ("test.xxx", None)



# © MNELAB developers
#
# License: BSD (3-clause)

from PySide6.QtGui import QKeySequence
from PySide6.QtWidgets import QGridLayout, QLabel, QSizePolicy, QVBoxLayout, QWidget

dev_label = (
    '<p align="right"><font color="red"><small>Development Version</small></font></p>'
)


def _make_shortcuts_table(actions):
    text_color = "#777"
    html = f"""<!DOCTYPE html>
    <html>
      <head>
        <style>
          html {{ font-size: 16px; }}
          kbd {{
            font-weight: 600;
            color: {text_color};
          }}
          table {{ color: {text_color}; }}
        </style>
      </head>
      <body>
        <table>
          <tbody>"""
    for action in actions:
        name = action.text().replace("&", "").replace(".", "")
        shortcut = action.shortcut().toString(format=QKeySequence.NativeText)
        modifier, key = shortcut[:-1].strip(), shortcut[-1]
        html += f'\n            <tr><td align="right" width="50%">{name} </td>'
        if modifier[-1] == "+":
            html += f"<td><kbd>{modifier[:-1]}</kbd>+"
        else:
            html += f"<td><kbd>{modifier}</kbd> "
        html += f"<kbd>{key}</kbd></td></tr>"
    html += """\n          </tbody>
        </table>
      </body>
    </html>"""
    return html


class InfoWidget(QWidget):
    """Display basic file information in a table (two columns).

    Parameters
    ----------
    values : dict
        Each key/value pair in this dict is displayed in a row separated by a colon.
    """

    def __init__(self, values=None):
        from .. import __version__

        super().__init__()
        vbox = QVBoxLayout(self)
        self.grid = QGridLayout()
        self.grid.setColumnStretch(1, 1)
        vbox.addLayout(self.grid)
        vbox.addStretch(1)
        self.set_values(values)
        if __version__.split(".")[-1].startswith("dev"):
            vbox.addStretch()
            vbox.addWidget(QLabel(dev_label))

    def set_values(self, values=None):
        """Set values (and overwrite existing values).

        Parameters
        ----------
        values : dict
            Each key/value pair in this dict is displayed in a row separated by a colon.
        """
        self.clear()
        if values:
            for row, (key, value) in enumerate(values.items()):
                left = QLabel(str(key) + ":")
                right = QLabel(str(value))
                right.setSizePolicy(QSizePolicy.Ignored, QSizePolicy.Fixed)
                self.grid.addWidget(left, row, 0)
                self.grid.addWidget(right, row, 1)

    def clear(self):
        """Clear all values."""
        item = self.grid.takeAt(0)
        while item:
            item.widget().deleteLater()
            del item
            item = self.grid.takeAt(0)


class EmptyWidget(QWidget):
    def __init__(self, actions):
        from .. import __version__

        super().__init__()
        text = QLabel(_make_shortcuts_table(actions))
        vbox = QVBoxLayout(self)
        vbox.addStretch()
        vbox.addWidget(text)
        vbox.addStretch()
        if __version__.split(".")[-1].startswith("dev"):
            vbox.addWidget(QLabel(dev_label))



# © MNELAB developers
#
# License: BSD (3-clause)

import sys

from PySide6.QtGui import QFont, QGuiApplication
from PySide6.QtWidgets import (
    QDialog,
    QDialogButtonBox,
    QPlainTextEdit,
    QPushButton,
    QVBoxLayout,
)

from mnelab.utils import PythonHighlighter


class HistoryDialog(QDialog):
    def __init__(self, parent, history):
        super().__init__(parent=parent)
        self.setWindowTitle("History")
        layout = QVBoxLayout()
        text = QPlainTextEdit()
        font = QFont()
        if sys.platform.startswith("darwin"):
            fontname = "menlo"
        elif sys.platform.startswith("win32"):
            fontname = "consolas"
        else:
            fontname = "monospace"
        font.setFamily(fontname)
        font.setStyleHint(QFont.Monospace)
        text.setFont(font)
        highlighter = PythonHighlighter(text.document())  # noqa: F841
        text.setReadOnly(True)
        text.setPlainText(history)
        layout.addWidget(text)
        buttonbox = QDialogButtonBox(QDialogButtonBox.Ok)
        clipboardbutton = QPushButton("Copy to clipboard")
        buttonbox.addButton(clipboardbutton, QDialogButtonBox.ActionRole)
        clipboard = QGuiApplication.clipboard()
        clipboardbutton.clicked.connect(lambda: clipboard.setText(history + "\n"))
        layout.addWidget(buttonbox)
        self.setLayout(layout)
        buttonbox.accepted.connect(self.accept)
        self.resize(700, 500)



import re

from mne import create_info
from mne.io import BaseRaw
from numpy import atleast_2d
from scipy.io import loadmat


class RawMAT(BaseRaw):
    """Raw data from .mat file."""

    def __init__(self, fname, variable, fs, transpose=False):
        """Read raw data from .mat file.

        Parameters
        ----------
        fname : str
            File name to load.
        variable : str
            Name of the variable to use. If nested within a struct, separate all names
            with dots. For example, `y.X` corresponds to a variable `X` contained in a
            struct `y`. If the cell array has multiple items, use `y.[0].X` to access
            the first item named `X`, `y.[1].X` to access the second item named `X`, and
            so on.
        fs : float
            Sampling frequency (in Hz).
        transpose : bool
            Whether to transpose the data; set to `True` if the original shape is *not*
            (channels, samples).
        """
        mat = loadmat(fname, simplify_cells=True)
        data = atleast_2d(_get_dict_value(mat, variable.split(".")))
        if transpose:
            data = data.T
        info = create_info(data.shape[0], fs, "eeg")
        super().__init__(preload=data, info=info, filenames=[fname])


def read_raw_mat(fname, variable, fs, transpose=False, *args, **kwargs):
    """Read raw data from .mat file.

    Parameters
    ----------
    fname : str
        File name to load.
    variable : str
        Name of the variable to use. If nested within a struct, separate all names with
        dots. For example, `y.X` corresponds to a variable `X` contained in a struct
        `y`. If the cell array has multiple items, use `y.[0].X` to access the first
        item named `X`, `y.[1].X` to access the second item named `X`, and so on.
    fs : float
        Sampling frequency (in Hz).
    transpose : bool
        Whether to transpose the data, the data should be of shape (channels, samples).

    Returns
    -------
    RawMAT
        The raw data.
    """
    return RawMAT(fname, variable, fs, transpose)


def parse_mat(fname):
    """Remove dunder variables from dict returned by scipy.io.loadmat()."""
    mat = loadmat(fname, simplify_cells=True)
    return {
        k: v for k, v in mat.items() if not k.startswith("__") and not k.endswith("__")
    }


def _get_dict_value(d, keys):
    """Get dictionary value from nested dictionary keys."""
    if isinstance(keys, str):  # no nesting
        return d[keys]
    value = d
    for key in keys:
        if match := re.search(r"\[(\d+)\]", key):  # list element
            value = value[int(match.group(1))]
        else:  # dict element
            value = value[key]
    return value



# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a
# full list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

project = "MNELAB"
copyright = "MNELAB Developers"

extensions = ["myst_parser"]

source_suffix = {
    ".rst": "restructuredtext",
    ".md": "markdown",
}

html_theme = "furo"
html_title = project
html_logo = "../../mnelab/images/mnelab_logo.png"
html_theme_options = {
    "sidebar_hide_name": True,
    "top_of_page_button": None,
}

templates_path = ["_templates"]

html_static_path = ["_static"]
html_css_files = ["custom.css"]
html_favicon = "_static/favicon.png"



import struct
import xml.etree.ElementTree as ETree
from collections import defaultdict

import mne
import numpy as np
import scipy.signal
from mne.io import BaseRaw, get_channel_type_constants
from pyxdf import load_xdf
from pyxdf.pyxdf import _read_varlen_int, open_xdf


class RawXDF(BaseRaw):
    """Raw data from .xdf file."""

    def __init__(
        self, fname, stream_ids, marker_ids=None, prefix_markers=False, fs_new=None
    ):
        """Read raw data from .xdf file.

        Parameters
        ----------
        fname : str
            File name to load.
        stream_ids : list[int]
            IDs of streams to load. A list of available streams can be obtained with
            `pyxdf.resolve_streams(fname)`.
        marker_ids : list[int] | None
            IDs of marker streams to load. If `None`, load all marker streams. A marker
            stream is a stream with a nominal sampling frequency of 0 Hz.
        prefix_markers : bool
            Whether to prefix marker streams with their corresponding stream ID.
        fs_new : float | None
            Resampling target frequency in Hz. If only one stream_id is given, this can
            be `None`, in which case no resampling is performed.
        """
        if len(stream_ids) > 1 and fs_new is None:
            raise ValueError(
                "Argument `fs_new` is required when reading multiple streams."
            )

        streams, _ = load_xdf(fname)
        streams = {stream["info"]["stream_id"]: stream for stream in streams}

        if all(_is_markerstream(streams[stream_id]) for stream_id in stream_ids):
            raise RuntimeError(
                "Loading only marker streams is not supported, at least one stream must"
                " be a regular stream."
            )

        labels_all, types_all, units_all = [], [], []
        channel_types = get_channel_type_constants(True)
        for stream_id in stream_ids:
            stream = streams[stream_id]

            n_chans = int(stream["info"]["channel_count"][0])
            labels, types, units = [], [], []
            try:
                for ch in stream["info"]["desc"][0]["channels"][0]["channel"]:
                    labels.append(str(ch["label"][0]))
                    if ch["type"] and ch["type"][0].lower() in channel_types:
                        types.append(ch["type"][0].lower())
                    else:
                        types.append("misc")
                    units.append(ch["unit"][0] if ch["unit"] else "NA")
            except (TypeError, IndexError):  # no channel labels found
                pass
            if not labels:
                labels = [f"{stream['info']['name'][0]}_{n}" for n in range(n_chans)]
            if not units:
                units = ["NA" for _ in range(n_chans)]
            if not types:
                types = ["misc" for _ in range(n_chans)]
            labels_all.extend(labels)
            types_all.extend(types)
            units_all.extend(units)

        if fs_new is not None:
            data, first_time = _resample_streams(streams, stream_ids, fs_new)
            fs = fs_new
        else:  # only possible if a single stream was selected
            data = streams[stream_ids[0]]["time_series"]
            first_time = streams[stream_ids[0]]["time_stamps"][0]
            fs = float(np.array(stream["info"]["effective_srate"]).item())

        info = mne.create_info(ch_names=labels_all, sfreq=fs, ch_types=types_all)

        microvolts = ("microvolt", "microvolts", "µV", "μV", "uV")
        scale = np.array([1e-6 if u in microvolts else 1 for u in units_all])
        data = (data * scale).T

        super().__init__(preload=data, info=info, filenames=[fname])

        # convert marker streams to annotations
        for stream_id, stream in streams.items():
            if marker_ids is not None and stream_id not in marker_ids:
                continue
            if not _is_markerstream(stream):
                continue
            onsets = stream["time_stamps"] - first_time
            prefix = f"{stream_id}-" if prefix_markers else ""
            descriptions = [
                f"{prefix}{item}" for sub in stream["time_series"] for item in sub
            ]
            self.annotations.append(onsets, [0] * len(onsets), descriptions)


def _resample_streams(streams, stream_ids, fs_new):
    """
    Resample multiple XDF streams to a given frequency.

    Parameters
    ----------
    streams : dict
        A dictionary mapping stream IDs to XDF streams.
    stream_ids : list[int]
        The IDs of the desired streams.
    fs_new : float
        Resampling target frequency in Hz.

    Returns
    -------
    all_time_series : np.ndarray
        Array of shape (n_samples, n_channels) containing raw data. Time intervals where
        a stream has no data contain `np.nan`.
    first_time : float
        Time of the very first sample in seconds.
    """
    start_times = []
    end_times = []
    n_total_chans = 0
    for stream_id in stream_ids:
        start_times.append(streams[stream_id]["time_stamps"][0])
        end_times.append(streams[stream_id]["time_stamps"][-1])
        n_total_chans += int(streams[stream_id]["info"]["channel_count"][0])
    first_time = min(start_times)
    last_time = max(end_times)

    n_samples = int(np.ceil((last_time - first_time) * fs_new))
    all_time_series = np.full((n_samples, n_total_chans), np.nan)

    col_start = 0
    for stream_id in stream_ids:
        start_time = streams[stream_id]["time_stamps"][0]
        end_time = streams[stream_id]["time_stamps"][-1]
        len_new = int(np.ceil((end_time - start_time) * fs_new))

        x_old = streams[stream_id]["time_series"]
        x_new = scipy.signal.resample(x_old, len_new, axis=0)

        row_start = int(
            np.floor((streams[stream_id]["time_stamps"][0] - first_time) * fs_new)
        )
        row_end = row_start + x_new.shape[0]
        col_end = col_start + x_new.shape[1]
        all_time_series[row_start:row_end, col_start:col_end] = x_new

        col_start += x_new.shape[1]

    return all_time_series, first_time


def read_raw_xdf(
    fname,
    stream_ids,
    marker_ids=None,
    prefix_markers=False,
    fs_new=None,
    *args,
    **kwargs,
):
    """Read XDF file.

    Parameters
    ----------
    fname : str
        File name to load.
    stream_ids : list[int]
        IDs of streams to load. A list of available streams can be obtained with
        `pyxdf.resolve_streams(fname)`.
    marker_ids : list[int] | None
        IDs of marker streams to load. If `None`, load all marker streams. A marker
        stream is a stream with a nominal sampling frequency of 0 Hz.
    prefix_markers : bool
        Whether to prefix marker streams with their corresponding stream ID.
    fs_new : float | None
        Resampling target frequency in Hz. If only one stream_id is given, this can be
        `None`, in which case no resampling is performed.

    Returns
    -------
    RawXDF
        The raw data.
    """

    return RawXDF(fname, stream_ids, marker_ids, prefix_markers, fs_new)


def _is_markerstream(stream):
    srate = float(stream["info"]["nominal_srate"][0])
    n_chans = int(stream["info"]["channel_count"][0])
    return srate == 0 and n_chans == 1


def get_xml(fname):
    """Get XML stream headers and footers from all streams.

    Parameters
    ----------
    fname : str
        Name of the XDF file.

    Returns
    -------
    xml : dict
        XML stream headers and footers.
    """
    with open_xdf(fname) as f:
        xml = defaultdict(dict)
        while True:
            try:
                nbytes = _read_varlen_int(f)
            except EOFError:
                return xml
            tag = struct.unpack("<H", f.read(2))[0]
            if tag in [2, 3, 4, 6]:
                stream_id = struct.unpack("<I", f.read(4))[0]
                if tag in [2, 6]:  # parse StreamHeader/StreamFooter chunk
                    string = f.read(nbytes - 6).decode()
                    xml[stream_id][tag] = ETree.fromstring(string)
                else:  # skip remaining chunk contents
                    f.seek(nbytes - 6, 1)
            else:
                f.seek(nbytes - 2, 1)  # skip remaining chunk contents


def list_chunks(fname):
    """List all chunks contained in an XDF file.

    Listing chunks summarizes the content of the XDF file. Because this function does
    not attempt to parse the data, this also works for corrupted files.

    Parameters
    ----------
    fname : str
        Name of the XDF file.

    Returns
    -------
    chunks : list
        List of dicts containing a short summary for each chunk.
    """
    with open_xdf(fname) as f:
        chunks = []
        while True:
            try:
                nbytes = _read_varlen_int(f)
            except EOFError:
                return chunks
            chunk = {"nbytes": nbytes}
            tag = struct.unpack("<H", f.read(2))[0]
            chunk["tag"] = tag
            if tag == 1:
                chunk["content"] = f.read(nbytes - 2).decode()
            elif tag == 5:
                chunk["content"] = (
                    "0x43 0xA5 0x46 0xDC 0xCB 0xF5 0x41 0x0F "
                    "0xB3 0x0E 0xD5 0x46 0x73 0x83 0xCB 0xE4"
                )
                f.seek(chunk["nbytes"] - 2, 1)  # skip remaining chunk contents
            elif tag in [2, 6]:  # XML
                chunk["stream_id"] = struct.unpack("<I", f.read(4))[0]
                chunk["content"] = (
                    f.read(chunk["nbytes"] - 6).decode().replace("\t", "  ")
                )
            elif tag == 4:
                chunk["stream_id"] = struct.unpack("<I", f.read(4))[0]
                collection_time = struct.unpack("<d", f.read(8))[0]
                offset_value = struct.unpack("<d", f.read(8))[0]
                chunk["content"] = (
                    f"Collection time: {collection_time}\n"
                    f"Offset value: {offset_value}"
                )
            elif tag == 3:
                chunk["stream_id"] = struct.unpack("<I", f.read(4))[0]
                remainder = chunk["nbytes"] - 6
                chunk["content"] = f"<BINARY DATA ({remainder} Bytes)>"
                f.seek(remainder, 1)  # skip remaining chunk contents
            else:
                f.seek(chunk["nbytes"] - 2, 1)  # skip remaining chunk contents
            chunks.append(chunk)



# © MNELAB developers
#
# License: BSD (3-clause)

from pathlib import Path

import mne
import numpy as np
from numpy.rec import fromarrays
from scipy.io import savemat


def write_fif(fname, raw):
    raw.save(fname, overwrite=True)


def write_set(fname, raw):
    """Export raw to EEGLAB .set file."""
    data = raw.get_data() * 1e6  # convert to microvolts
    fs = raw.info["sfreq"]
    times = raw.times
    ch_names = raw.info["ch_names"]
    chanlocs = fromarrays([ch_names], names=["labels"])
    events = fromarrays(
        [
            raw.annotations.description,
            raw.annotations.onset * fs + 1,
            raw.annotations.duration * fs,
        ],
        names=["type", "latency", "duration"],
    )
    savemat(
        fname,
        dict(
            EEG=dict(
                data=data,
                setname=fname,
                nbchan=data.shape[0],
                pnts=data.shape[1],
                trials=1,
                srate=fs,
                xmin=times[0],
                xmax=times[-1],
                chanlocs=chanlocs,
                event=events,
                icawinv=[],
                icasphere=[],
                icaweights=[],
            )
        ),
        appendmat=False,
    )


def write_edf(fname, raw):
    """Export raw to EDF file."""
    raw.export(fname)


def write_bv(fname, raw, events=None):
    """Export data to BrainVision EEG/VHDR/VMRK file (requires pybv)."""
    import pybv

    name, _ = Path(fname).stem, "".join(Path(fname).suffixes)
    parent = Path(fname).parent
    data = raw.get_data()
    fs = raw.info["sfreq"]
    ch_names = raw.info["ch_names"]
    if events is None:
        if raw.annotations:
            events = mne.events_from_annotations(raw, regexp=None)[0]
            dur = raw.annotations.duration * fs
            events = np.column_stack([events[:, [0, 2]], dur.astype(int)])
    else:
        events = events[:, [0, 2]]
    units = [ch["unit"] for ch in raw.info["chs"]]
    units = ["µV" if unit == 107 else "AU" for unit in units]
    pybv.write_brainvision(
        data=data,
        sfreq=fs,
        ch_names=ch_names,
        fname_base=name,
        folder_out=parent,
        events=events,
        unit=units,
    )


# this dict contains each supported file extension as a key; the corresponding value is
# a list with three elements: (1) the writer function, (2) the full file format name,
# and (3) a (comma-separated) string indicating the supported objects
writers = {
    ".edf": [write_edf, "European Data Format", "raw"],
    ".fif": [write_fif, "Elekta Neuromag", "raw,epoch"],
    ".fif.gz": [write_fif, "Elekta Neuromag", "raw,epoch"],
    ".set": [write_set, "EEGLAB", "raw"],
    ".eeg": [write_bv, "BrainVision", "raw"],
}


def write_raw(fname, raw):
    maxsuffixes = max([ext.count(".") for ext in writers.keys()])
    suffixes = Path(fname).suffixes
    for i in range(-maxsuffixes, 0):
        ext = "".join(suffixes[i:])
        if ext in writers.keys():
            return writers[ext][0](fname, raw)
    else:
        raise ValueError(f"Unknown file type '{suffixes}'.")



# © MNELAB developers
#
# License: BSD (3-clause)

from mnelab.utils.dependencies import have
from mnelab.utils.syntax import PythonHighlighter
from mnelab.utils.utils import count_locations, image_path, natural_sort



from PySide6.QtWidgets import (
    QCheckBox,
    QDialog,
    QDialogButtonBox,
    QDoubleSpinBox,
    QGridLayout,
    QLabel,
    QVBoxLayout,
)


class NpyDialog(QDialog):
    def __init__(self, parent, shape):
        super().__init__(parent)

        self.setWindowTitle("Import NumPy array")

        vbox = QVBoxLayout(self)
        grid = QGridLayout()
        grid.addWidget(QLabel("Array shape:"), 0, 0)
        grid.addWidget(QLabel(f"{' × '.join(map(str, shape))}"), 0, 1)
        grid.addWidget(QLabel("Sampling frequency:"), 1, 0)

        self._fs = QDoubleSpinBox()
        self._fs.setRange(0, 20e3)
        self._fs.setValue(250)
        self._fs.setSuffix(" Hz")
        grid.addWidget(self._fs, 1, 1)

        self._transpose = QCheckBox("Transpose")
        if shape[0] > shape[1]:  # transpose if there are more rows than columns
            self._transpose.setChecked(True)
        else:
            self._transpose.setChecked(False)
        grid.addWidget(self._transpose, 2, 0)

        vbox.addLayout(grid)
        buttonbox = QDialogButtonBox(QDialogButtonBox.Ok | QDialogButtonBox.Cancel)
        vbox.addWidget(buttonbox)
        buttonbox.accepted.connect(self.accept)
        buttonbox.rejected.connect(self.reject)
        vbox.setSizeConstraint(QVBoxLayout.SetFixedSize)

    @property
    def fs(self):
        return self._fs.value()

    @property
    def transpose(self):
        return self._transpose.isChecked()
